# INITIALIZE

# set the path where the functions are located
path_f.chv <- #...

# set the path where the gene-set workspace is located
path_w.chv <- #...

setwd (path_f.chv)
source ("Fisher_Enrichment.R")
source ("Fisher_Enrichment_FDR.R")

setwd (path_w.chv)
load ("Core01_GO_Pathways_PFAM_Hs_eg.RData")

# assign your set of genes to be tested
test.genes <- # ...

# COMPUTE

# Filter

# MAX and MIN respectively
thU.n <- 900
thL.n <-  10

f.filter <- function (gs.ls)
	{
	gs.ls      <- lapply (gs.ls, unique)	
	lengths.nv <- unlist (lapply (gs.ls, length))
	gs_f.ls    <- gs.ls[lengths.nv <= thU.n & lengths.nv >= thL.n]
	return (gs_f.ls)
	}

gs_f.ls <- f.filter (gs_GO_Pws_PFAM_Hs_eg.ls)
gs_names.chv <- gs_GO_Pws_PFAM_Hs_ID2name.chv

# Universe

# In absence of a universe such as an array, use the following
Uni.genes <- unique (unlist (gs_f.ls))

# This test.gene assignment can be used for testing purposes
# test.genes <- gs_f.ls[["GO:0006915"]]

# Test

enr.df <- f.Enr_Fisher (
 			GS.ls        = gs_f.ls, 
 			GS_names.chv = gs_names.chv,
 			exp.genes    = test.genes, 
 			uni.genes    = Uni.genes
 			)

# FDR 
# - mind fdr estimation will take a while

enr.df$FDR <- f.Fisher_FDR (
					GS.ls       = gs_f.ls, 
					exp.genes   = test.genes, 
					uni.genes   = Uni.genes, 
					pval_obs.nv = enr.df$pvalue, 
					iter.n      = 2000
					)

# FORMAT FOR ENRICHMENT MAP

# Enrichment Table

enr.df$Pheno <- rep (+1, nrow (enr.df))

sel_col.names <- c ("GS_ID", "GS_name", "pvalue", "FDR", "Pheno")

write.table (
	enr_f.df[, sel_col.names],
	sep  = "\t",
	col.names = T,
	row.names = F,
	file = "Enr_FET.txt",
	)
	
# GMT

f.collapse <- function (input.genes)
        {paste (input.genes, collapse = "\t")}

genes.chv  <- sapply (gs_f.ls, f.collapse)
output.chv <- paste (
				names (gs_f.ls), 
				gs_names.chv[names (gs_f.ls)], 
				genes.chv, sep = "\t"
				)

cat (output.chv, sep = "\n", file = "GO_KEGG_Pws_PF_Hs_eg.GMT")
		