f.Fisher.enr <- function (
			GOf.list, 
			GOterm.chv,
			sample.Entrez, 
			uni.Entrez)

# updated to new GO package

{

# DEFINING CORE OUTPUT VARIABLE
# 	(ENRICHMENT.DF)

row.n <- length (GOf.list)

# columns of < enrichment.df > are: 
# - GO ID, 
# - term name, 
# - number of GO term genes in the universe ("SplG"), 
# - number of GO term genes in the sample ("UniG"), 
# - uncorrected p-value
# - ratio = (sample.GOterm.length / sample.tot.length) / (uni.GOterm.length / uni.tot.length)

#debug(1)
#print("OK 1")

enrichment.df <- data.frame (GO.ID     = character (row.n),
			     GO.term   = character (row.n),
			     Tot       = integer (row.n),
			     Enr       = integer (row.n),
			     p.val     = numeric (row.n),
			     ratio     = numeric (row.n),
			     stringsAsFactors = F)


# COMPUTING ENRICHMENT.DF

for (i in 1: row.n)
	{

	GO.ID <- names (GOf.list)[i]

	GO.Gene <- unique (GOf.list[[i]])

	enr.n    <- length (intersect(sample.Entrez, GO.Gene))
	GOtot.n  <- length (intersect(uni.Entrez, GO.Gene))
	sample.n <- length (sample.Entrez)
	uni.n    <- length (uni.Entrez)

	table.mx <- matrix (
					ncol = 2, 
					nrow = 2, 
					data = c (
						enr.n, 
						GOtot.n - enr.n, 
						sample.n - enr.n, 
						uni.n - (GOtot.n + sample.n - enr.n)
						), 
					byrow = T
					)

	p.value <- fisher.test (table.mx, alternative = "greater")$p.value

	ratio.n <- (enr.n / sample.n) / (GOtot.n / uni.n)

	#debug(2)
	#print("OK 2")
	#print(paste(i,names(GOf.list)[i]))

	enrichment.df$Enr[i]     <- enr.n
	enrichment.df$Tot[i]     <- GOtot.n
	enrichment.df$p.val[i]   <- (signif (p.value, 3))
	enrichment.df$ratio[i]   <- (signif (ratio.n, 3))

	}

enrichment.df$GO.ID   <- names (GOf.list)
enrichment.df$GO.term <- GOterm.chv[enrichment.df$GO.ID]


# sorting by increasing p-value

enrichment.df <- enrichment.df[order (enrichment.df$p.val, decreasing = F), ]

return (enrichment.df)

}                                                                                                      