f.Fisher.enr <- function (
			GS.ls, 
			GS_names.chv,
			exp.genes, 
			uni.genes)

# updated to new GO package

{

# DEFINING CORE OUTPUT VARIABLE
# 	(ENRICHMENT.DF)

row.n <- length (GS.ls)

# columns of < enrichment.df > are: 
# - GO ID, 
# - term name, 
# - number of GO term genes in the universe ("SplG"), 
# - number of GO term genes in the sample ("UniG"), 
# - uncorrected p-value
# - ratio = (sample.GOterm.length / sample.tot.length) / (uni.GOterm.length / uni.tot.length)

#debug(1)
#print("OK 1")

enrichment.df <- data.frame (
				 	GS_ID     = character (row.n),
			     	GS_Name   = character (row.n),
			     	Tot       = integer (row.n),
			     	Enr       = integer (row.n),
			     	pvalue    = numeric (row.n),
			     	ratio     = numeric (row.n),
			     	stringsAsFactors = F)


# COMPUTING ENRICHMENT.DF

for (i in 1: row.n)
	{

	GS_ID <- names (GS.ls)[i]

	GO.Gene <- unique (GS.ls[[i]])

	enr.n    <- length (intersect(exp.genes, GO.Gene))
	GOtot.n  <- length (intersect(uni.genes, GO.Gene))
	sample.n <- length (exp.genes)
	uni.n    <- length (uni.genes)

	table.mx <- matrix (
					ncol = 2, 
					nrow = 2, 
					data = c (
						enr.n, 
						GOtot.n - enr.n, 
						sample.n - enr.n, 
						uni.n - (GOtot.n + sample.n - enr.n)
						), 
					byrow = T
					)

	p.value <- fisher.test (table.mx, alternative = "greater")$p.value

	ratio.n <- (enr.n / sample.n) / (GOtot.n / uni.n)

	#debug(2)
	#print("OK 2")
	#print(paste(i,names(GS.ls)[i]))

	enrichment.df$Enr[i]     <- enr.n
	enrichment.df$Tot[i]     <- GOtot.n
	enrichment.df$pvalue[i]  <- (signif (p.value, 3))
	enrichment.df$ratio[i]   <- (signif (ratio.n, 3))

	}

enrichment.df$GS_ID   <- names (GS.ls)
enrichment.df$GS_Name <- GS_names.chv[enrichment.df$GS_ID]


# sorting by increasing p-value

enrichment.df <- enrichment.df[order (enrichment.df$pvalue, decreasing = F), ]

return (enrichment.df)

}                                                                                                      